@@ -23,9 +23,12 @@
 #include "agentbase.h"
 #include "agentbase_p.h"
 
+#include "agentmanager.h"
 #include "changerecorder.h"
 #include "controladaptor.h"
+#include "dbusconnectionpool.h"
 #include "itemfetchjob.h"
+#include "kdepimlibs-version.h"
 #include "monitor_p.h"
 #include "servermanager_p.h"
 #include "session.h"
@@ -39,6 +42,8 @@
 #include <klocale.h>
 #include <kstandarddirs.h>
 
+#include <Solid/PowerManagement>
+
 #include <QtCore/QDir>
 #include <QtCore/QSettings>
 #include <QtCore/QTimer>
@@ -48,6 +53,9 @@
 #include <signal.h>
 #include <stdlib.h>
 
+
+//#define EXPERIMENTAL_INPROCESS_AGENTS 1
+
 using namespace Akonadi;
 
 static AgentBase *sAgentBase = 0;
@@ -118,16 +126,24 @@ void AgentBase::ObserverV2::itemLinked( const Akonadi::Item& item, const Akonadi
 {
   Q_UNUSED( item );
   Q_UNUSED( collection );
-  if ( sAgentBase != 0 )
+  if ( sAgentBase != 0 ) {
+    // not implementation, let's disconnect the signal to enable optimizations in Monitor
+    QObject::disconnect( sAgentBase->changeRecorder(), SIGNAL(itemLinked(Akonadi::Item,Akonadi::Collection)),
+                         sAgentBase->d_ptr, SLOT(itemLinked(Akonadi::Item,Akonadi::Collection)) );
     sAgentBase->d_ptr->changeProcessed();
+  }
 }
 
 void AgentBase::ObserverV2::itemUnlinked( const Akonadi::Item& item, const Akonadi::Collection& collection )
 {
   Q_UNUSED( item );
   Q_UNUSED( collection );
-  if ( sAgentBase != 0 )
+  if ( sAgentBase != 0 ) {
+    // not implementation, let's disconnect the signal to enable optimizations in Monitor
+    QObject::disconnect( sAgentBase->changeRecorder(), SIGNAL(itemUnlinked(Akonadi::Item,Akonadi::Collection)),
+                         sAgentBase->d_ptr, SLOT(itemUnlinked(Akonadi::Item,Akonadi::Collection)) );
     sAgentBase->d_ptr->changeProcessed();
+  }
 }
 
 void AgentBase::ObserverV2::collectionMoved( const Akonadi::Collection &collection, const Akonadi::Collection &source, const Akonadi::Collection &dest )
@@ -149,6 +165,7 @@ void AgentBase::ObserverV2::collectionChanged( const Akonadi::Collection &collec
 
 AgentBasePrivate::AgentBasePrivate( AgentBase *parent )
   : q_ptr( parent ),
+    mDBusConnection( QString() ),
     mStatusCode( AgentBase::Idle ),
     mProgress( 0 ),
     mNeedsNetwork( false ),
@@ -156,6 +173,9 @@ AgentBasePrivate::AgentBasePrivate( AgentBase *parent )
     mSettings( 0 ),
     mObserver( 0 )
 {
+#ifdef Q_OS_WINCE
+  QThread::currentThread()->setPriority(QThread::LowPriority);
+#endif
   Internal::setClientType( Internal::Agent );
 }
 
@@ -174,13 +194,19 @@ void AgentBasePrivate::init()
    */
   SessionPrivate::createDefaultSession( mId.toLatin1() );
 
-  mTracer = new org::freedesktop::Akonadi::Tracer( QLatin1String( "org.freedesktop.Akonadi" ), QLatin1String( "/tracing" ),
-                                           QDBusConnection::sessionBus(), q );
+  if ( QThread::currentThread() != QCoreApplication::instance()->thread() ) {
+    mDBusConnection = QDBusConnection::connectToBus( QDBusConnection::SessionBus, q->identifier() );
+    Q_ASSERT( mDBusConnection.isConnected() );
+  }
+
+  mTracer = new org::freedesktop::Akonadi::Tracer( QLatin1String( "org.freedesktop.Akonadi" ),
+                                                   QLatin1String( "/tracing" ),
+                                                   DBusConnectionPool::threadConnection(), q );
 
   new Akonadi__ControlAdaptor( q );
   new Akonadi__StatusAdaptor( q );
-  if ( !QDBusConnection::sessionBus().registerObject( QLatin1String( "/" ), q, QDBusConnection::ExportAdaptors ) )
-    q->error( QString::fromLatin1( "Unable to register object at dbus: %1" ).arg( QDBusConnection::sessionBus().lastError().message() ) );
+  if ( !DBusConnectionPool::threadConnection().registerObject( QLatin1String( "/" ), q, QDBusConnection::ExportAdaptors ) )
+    q->error( QString::fromLatin1( "Unable to register object at dbus: %1" ).arg( DBusConnectionPool::threadConnection().lastError().message() ) );
 
   mSettings = new QSettings( QString::fromLatin1( "%1/agent_config_%2" ).arg( XdgBaseDirs::saveDir( "config", QLatin1String( "akonadi" ) ), mId ), QSettings::IniFormat );
 
@@ -203,54 +229,76 @@ void AgentBasePrivate::init()
     }
   }
 
-  connect( mChangeRecorder, SIGNAL( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ),
-           SLOT( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ),
-           SLOT( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ) );
-  connect( mChangeRecorder, SIGNAL( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
-           SLOT( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( itemRemoved( const Akonadi::Item& ) ),
-           SLOT( itemRemoved( const Akonadi::Item& ) ) );
-  connect( mChangeRecorder, SIGNAL( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ),
-           SLOT( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
-           SLOT( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
-           SLOT( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( collectionChanged( const Akonadi::Collection& ) ),
-           SLOT( collectionChanged( const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ),
-           SLOT( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ) );
-  connect( mChangeRecorder, SIGNAL( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
-           SLOT( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  connect( mChangeRecorder, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
-           SLOT( collectionRemoved( const Akonadi::Collection& ) ) );
+  connect( mChangeRecorder, SIGNAL(itemAdded(Akonadi::Item,Akonadi::Collection)),
+           SLOT(itemAdded(Akonadi::Item,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(itemChanged(Akonadi::Item,QSet<QByteArray>)),
+           SLOT(itemChanged(Akonadi::Item,QSet<QByteArray>)) );
+  connect( mChangeRecorder, SIGNAL(itemMoved(Akonadi::Item,Akonadi::Collection,Akonadi::Collection)),
+           SLOT(itemMoved(Akonadi::Item,Akonadi::Collection,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(itemRemoved(Akonadi::Item)),
+           SLOT(itemRemoved(Akonadi::Item)) );
+  connect( mChangeRecorder, SIGNAL(collectionAdded(Akonadi::Collection,Akonadi::Collection)),
+           SLOT(collectionAdded(Akonadi::Collection,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(itemLinked(Akonadi::Item,Akonadi::Collection)),
+           SLOT(itemLinked(Akonadi::Item,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(itemUnlinked(Akonadi::Item,Akonadi::Collection)),
+           SLOT(itemUnlinked(Akonadi::Item,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(collectionChanged(Akonadi::Collection)),
+           SLOT(collectionChanged(Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(collectionChanged(Akonadi::Collection,QSet<QByteArray>)),
+           SLOT(collectionChanged(Akonadi::Collection,QSet<QByteArray>)) );
+  connect( mChangeRecorder, SIGNAL(collectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)),
+           SLOT(collectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(collectionRemoved(Akonadi::Collection)),
+           SLOT(collectionRemoved(Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(collectionSubscribed(Akonadi::Collection,Akonadi::Collection)),
+           SLOT(collectionSubscribed(Akonadi::Collection,Akonadi::Collection)) );
+  connect( mChangeRecorder, SIGNAL(collectionUnsubscribed(Akonadi::Collection)),
+           SLOT(collectionUnsubscribed(Akonadi::Collection)) );
 
-  connect( q, SIGNAL( status( int, const QString& ) ), q, SLOT( slotStatus( int, const QString& ) ) );
-  connect( q, SIGNAL( percent( int ) ), q, SLOT( slotPercent( int ) ) );
-  connect( q, SIGNAL( warning( const QString& ) ), q, SLOT( slotWarning( const QString& ) ) );
-  connect( q, SIGNAL( error( const QString& ) ), q, SLOT( slotError( const QString& ) ) );
+  connect( q, SIGNAL(status(int,QString)), q, SLOT(slotStatus(int,QString)) );
+  connect( q, SIGNAL(percent(int)), q, SLOT(slotPercent(int)) );
+  connect( q, SIGNAL(warning(QString)), q, SLOT(slotWarning(QString)) );
+  connect( q, SIGNAL(error(QString)), q, SLOT(slotError(QString)) );
+
+  connect( Solid::PowerManagement::notifier(), SIGNAL(resumingFromSuspend()), q, SLOT(slotResumedFromSuspend()) );
 
   // Use reference counting to allow agents to finish internal jobs when the
   // agent is stopped.
   KGlobal::ref();
-  KGlobal::setAllowQuit( true );
+  if ( QThread::currentThread() == QCoreApplication::instance()->thread() )
+    KGlobal::setAllowQuit( true );
 
+#ifndef Q_OS_WINCE
   // disable session management
   if ( KApplication::kApplication() )
     KApplication::kApplication()->disableSessionManagement();
+#endif
 
-  QTimer::singleShot( 0, q, SLOT( delayedInit() ) );
+  mResourceTypeName = AgentManager::self()->instance(mId).type().name();
+  setProgramName();
+
+  QTimer::singleShot( 0, q, SLOT(delayedInit()) );
 }
 
 void AgentBasePrivate::delayedInit()
 {
   Q_Q( AgentBase );
-  if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.freedesktop.Akonadi.Agent." ) + mId ) )
-    kFatal() << "Unable to register service at dbus:" << QDBusConnection::sessionBus().lastError().message();
+  if ( !DBusConnectionPool::threadConnection().registerService( QLatin1String( "org.freedesktop.Akonadi.Agent." ) + mId ) )
+    kFatal() << "Unable to register service at dbus:" << DBusConnectionPool::threadConnection().lastError().message();
   q->setOnline( mOnline );
 }
 
+void AgentBasePrivate::setProgramName()
+{
+  // ugly, really ugly, if you find another solution, change it and blame me for this code (Andras)
+  QString programName = mResourceTypeName;
+  if ( !mName.isEmpty() ) {
+    programName = i18nc( "Name and type of Akonadi resource", "%1 of type %2", mName, mResourceTypeName ) ;
+  }
+  const_cast<KAboutData*>( KGlobal::mainComponent().aboutData() )->setProgramName( ki18n( programName.toUtf8() ) );
+}
+
 void AgentBasePrivate::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
 {
   if ( mObserver != 0 )
@@ -378,10 +426,23 @@ void AgentBasePrivate::collectionRemoved( const Akonadi::Collection &collection
     mObserver->collectionRemoved( collection );
 }
 
+void AgentBasePrivate::collectionSubscribed( const Akonadi::Collection &collection, const Akonadi::Collection &parent )
+{
+  Q_UNUSED( collection );
+  Q_UNUSED( parent );
+  changeProcessed();
+}
+
+void AgentBasePrivate::collectionUnsubscribed( const Akonadi::Collection &collection )
+{
+  Q_UNUSED( collection );
+  changeProcessed();
+}
+
 void AgentBasePrivate::changeProcessed()
 {
   mChangeRecorder->changeProcessed();
-  QTimer::singleShot( 0, mChangeRecorder, SLOT( replayNext() ) );
+  QTimer::singleShot( 0, mChangeRecorder, SLOT(replayNext()) );
 }
 
 void AgentBasePrivate::slotStatus( int status, const QString &message )
@@ -435,6 +496,11 @@ void AgentBasePrivate::slotNetworkStatusChange( Solid::Networking::Status stat )
   q->setOnline( stat == Solid::Networking::Unknown || stat == Solid::Networking::Connected );
 }
 
+void AgentBasePrivate::slotResumedFromSuspend()
+{
+  if ( mNeedsNetwork )
+    slotNetworkStatusChange( Solid::Networking::status() );
+}
 
 AgentBase::AgentBase( const QString & id )
   : d_ptr( new AgentBasePrivate( this ) )
@@ -475,14 +541,11 @@ QString AgentBase::parseArguments( int argc, char **argv )
     exit( 1 );
   }
 
-  QByteArray catalog;
-  char *p = strrchr( argv[0], '/' );
-  if ( p )
-    catalog = QByteArray( p + 1 );
-  else
-    catalog = QByteArray( argv[0] );
+  const QFileInfo fi( QString::fromLocal8Bit( argv[0] ) );
+  // strip off full path and possible .exe suffix
+  const QByteArray catalog = fi.baseName().toLatin1();
 
-  KCmdLineArgs::init( argc, argv, identifier.toLatin1(), catalog, ki18n( "Akonadi Agent" ), "0.1",
+  KCmdLineArgs::init( argc, argv, identifier.toLatin1(), catalog, ki18n( "Akonadi Agent" ), KDEPIMLIBS_VERSION,
                       ki18n( "Akonadi Agent" ) );
 
   KCmdLineOptions options;
@@ -545,8 +608,10 @@ void AgentBase::setNeedsNetwork( bool needsNetwork )
 
   if ( d->mNeedsNetwork ) {
     connect( Solid::Networking::notifier()
-           , SIGNAL( statusChanged( Solid::Networking::Status ) )
-           , this, SLOT( slotNetworkStatusChange( Solid::Networking::Status ) ) );
+           , SIGNAL(statusChanged(Solid::Networking::Status))
+           , this, SLOT(slotNetworkStatusChange(Solid::Networking::Status))
+           , Qt::UniqueConnection );
+
   } else {
     disconnect( Solid::Networking::notifier(), 0, 0, 0 );
     setOnline( true );
@@ -557,6 +622,11 @@ void AgentBase::setOnline( bool state )
 {
   Q_D( AgentBase );
   d->mOnline = state;
+
+  const QString newMessage = d->defaultReadyMessage();
+  if ( d->mStatusMessage != newMessage && d->mStatusCode != AgentBase::Broken )
+    emit status( d->mStatusCode, newMessage );
+
   d->mSettings->setValue( QLatin1String( "Agent/Online" ), state );
   doSetOnline( state );
   emit onlineChanged( state );
@@ -582,7 +652,7 @@ void AgentBase::configure( qlonglong windowId )
 
 WId AgentBase::winIdForDialogs() const
 {
-  const bool registered = QDBusConnection::sessionBus().interface()->isServiceRegistered( QLatin1String( "org.freedesktop.akonaditray" ) );
+  const bool registered = DBusConnectionPool::threadConnection().interface()->isServiceRegistered( QLatin1String( "org.freedesktop.akonaditray" ) );
   if ( !registered )
     return 0;
 
@@ -646,7 +716,7 @@ void AgentBase::cleanup()
   /*
    * ... and also remove the agent configuration file if there is one.
    */
-  QString configFile = KStandardDirs::locateLocal( "config", KGlobal::config()->name() );
+  QString configFile = KStandardDirs::locateLocal( "config", config()->name() );
   QFile::remove( configFile );
 
   KGlobal::deref();
@@ -654,6 +724,7 @@ void AgentBase::cleanup()
 
 void AgentBase::registerObserver( Observer *observer )
 {
+  // TODO in theory we should re-connect change recorder signals here that we disconnected previously
   d_ptr->mObserver = observer;
 }
 
@@ -679,6 +750,8 @@ void AgentBase::setAgentName( const QString &name )
 
   d->mSettings->sync();
 
+  d->setProgramName();
+
   emit agentNameChanged( d->mName );
 }
 
@@ -702,6 +775,14 @@ ChangeRecorder * AgentBase::changeRecorder() const
   return d_ptr->mChangeRecorder;
 }
 
+KSharedConfigPtr AgentBase::config()
+{
+  if ( QCoreApplication::instance()->thread() == QThread::currentThread() )
+    return KGlobal::config();
+  else
+    return componentData().config();
+}
+
 void AgentBase::abort()
 {
   emit abortRequested();
@@ -712,5 +793,20 @@ void AgentBase::reconfigure()
   emit reloadConfiguration();
 }
 
+extern QThreadStorage<KComponentData*> s_agentComponentDatas;
+
+KComponentData AgentBase::componentData()
+{
+  if ( QThread::currentThread() == QCoreApplication::instance()->thread() ) {
+    if ( s_agentComponentDatas.hasLocalData() )
+      return *(s_agentComponentDatas.localData());
+    else
+      return KGlobal::mainComponent();
+  }
+
+  Q_ASSERT( s_agentComponentDatas.hasLocalData() );
+  return *(s_agentComponentDatas.localData());
+}
+
 #include "agentbase.moc"
 #include "agentbase_p.moc"
