@@ -25,11 +25,13 @@
 #include <QtCore/QHash>
 #include <QtCore/QMimeData>
 #include <QtCore/QTimer>
+#include <QtGui/QAbstractProxyModel>
 #include <QtGui/QApplication>
 #include <QtGui/QPalette>
 
 #include <KDE/KIcon>
 #include <KDE/KLocale>
+#include <KDE/KMessageBox>
 #include <KDE/KUrl>
 
 #include <akonadi/attributefactory.h>
@@ -46,9 +48,6 @@
 #include "kdebug.h"
 #include "pastehelper_p.h"
 
-// TODO:
-// * Implement ordering support.
-
 Q_DECLARE_METATYPE( QSet<QByteArray> )
 
 using namespace Akonadi;
@@ -69,20 +68,42 @@ EntityTreeModel::EntityTreeModel( ChangeRecorder *monitor,
   : QAbstractItemModel( parent ),
     d_ptr( d )
 {
-  d->init(monitor );
+  d->init( monitor );
 }
 
 EntityTreeModel::~EntityTreeModel()
 {
   Q_D( EntityTreeModel );
 
-  foreach( const QList<Node*> &list, d->m_childEntities ) {
-    qDeleteAll( list );
+  foreach ( const QList<Node*> &list, d->m_childEntities ) {
+    QList<Node*>::const_iterator it = list.constBegin();
+    const QList<Node*>::const_iterator end = list.constEnd();
+    for ( ; it != end; ++it ) {
+      delete *it;
+    }
   }
 
+  d->m_rootNode = 0;
+
   delete d_ptr;
 }
 
+bool EntityTreeModel::includeUnsubscribed() const
+{
+  Q_D( const EntityTreeModel );
+  return d->m_includeUnsubscribed;
+}
+
+void EntityTreeModel::setIncludeUnsubscribed( bool show )
+{
+  Q_D( EntityTreeModel );
+  d->beginResetModel();
+  d->m_includeUnsubscribed = show;
+  d->m_monitor->setAllMonitored( show );
+  d->endResetModel();
+}
+
+
 bool EntityTreeModel::systemEntitiesShown() const
 {
   Q_D( const EntityTreeModel );
@@ -122,7 +143,9 @@ QVariant EntityTreeModel::entityData( const Item &item, int column, int role ) c
              !item.attribute<EntityDisplayAttribute>()->displayName().isEmpty() ) {
           return item.attribute<EntityDisplayAttribute>()->displayName();
         } else {
-          return item.remoteId();
+          if (!item.remoteId().isEmpty())
+            return item.remoteId();
+          return QString(QLatin1String("<") + QString::number( item.id() ) + QLatin1String(">"));
         }
         break;
       case Qt::DecorationRole:
@@ -162,7 +185,9 @@ QVariant EntityTreeModel::entityData( const Collection &collection, int column,
              !collection.attribute<EntityDisplayAttribute>()->displayName().isEmpty() ) {
           return collection.attribute<EntityDisplayAttribute>()->displayName();
         }
-        return collection.name();
+        if ( !collection.name().isEmpty() )
+          return collection.name();
+        return i18n( "Loading..." );
       }
       break;
     case Qt::DecorationRole:
@@ -200,7 +225,7 @@ QVariant EntityTreeModel::data( const QModelIndex & index, int role ) const
 
   const Node *node = reinterpret_cast<Node *>( index.internalPointer() );
 
-  if ( ParentCollectionRole == role ) {
+  if ( ParentCollectionRole == role && d->m_collectionFetchStrategy != FetchNoCollections ) {
     const Collection parentCollection = d->m_collections.value( node->parent );
     Q_ASSERT( parentCollection.isValid() );
 
@@ -235,6 +260,19 @@ QVariant EntityTreeModel::data( const QModelIndex & index, int role ) const
       case EntityUrlRole:
         return collection.url().url();
         break;
+      case UnreadCountRole:
+      {
+        CollectionStatistics statistics = collection.statistics();
+        return statistics.unreadCount();
+      }
+      case FetchStateRole:
+      {
+        return d->m_pendingCollectionRetrieveJobs.contains(collection.id()) ? FetchingState : IdleState;
+      }
+      case CollectionSyncProgressRole:
+      {
+        return d->m_collectionSyncProgress.value( collection.id() );
+      }
       case Qt::BackgroundRole:
       {
         if ( collection.hasAttribute<EntityDisplayAttribute>() )
@@ -257,6 +295,8 @@ QVariant EntityTreeModel::data( const QModelIndex & index, int role ) const
       return QVariant();
 
     switch ( role ) {
+      case ParentCollectionRole:
+        return QVariant::fromValue( item.parentCollection() );
       case MimeTypeRole:
         return item.mimeType();
         break;
@@ -286,7 +326,7 @@ QVariant EntityTreeModel::data( const QModelIndex & index, int role ) const
         if ( item.hasAttribute<EntityDisplayAttribute>() )
         {
           EntityDisplayAttribute *eda = item.attribute<EntityDisplayAttribute>();
-          QColor color = eda->backgroundColor();
+          const QColor color = eda->backgroundColor();
           if ( color.isValid() )
             return color;
         }
@@ -352,12 +392,9 @@ Qt::ItemFlags EntityTreeModel::flags( const QModelIndex & index ) const
     // Rights come from the parent collection.
 
     Collection parentCollection;
-    if ( !index.parent().isValid() )
-    {
+    if ( !index.parent().isValid() ) {
       parentCollection = d->m_rootCollection;
-    }
-    else
-    {
+    } else {
       const Node *parentNode = reinterpret_cast<Node *>( index.parent().internalPointer() );
 
       parentCollection = d->m_collections.value( parentNode->id );
@@ -452,7 +489,7 @@ bool EntityTreeModel::dropMimeData( const QMimeData * data, Qt::DropAction actio
       foreach ( const KUrl &url, urls ) {
         const Collection collection = d->m_collections.value( Collection::fromUrl( url ).id() );
         if ( collection.isValid() ) {
-          if ( collection.parentCollection().id() == destCollection.id() ) {
+          if ( collection.parentCollection().id() == destCollection.id() && action != Qt::CopyAction) {
             kDebug() << "Error: source and destination of move are the same.";
             return false;
           }
@@ -461,10 +498,21 @@ bool EntityTreeModel::dropMimeData( const QMimeData * data, Qt::DropAction actio
             kDebug() << "unwanted collection" << mimeChecker.wantedMimeTypes() << collection.contentMimeTypes();
             return false;
           }
+
+          if ( url.hasQueryItem( QLatin1String( "name" ) ) ) {
+            const QString collectionName = url.queryItemValue( QLatin1String( "name" ) );
+            const QStringList collectionNames = d->childCollectionNames( destCollection );
+
+            if ( collectionNames.contains( collectionName ) ) {
+              KMessageBox::error( 0, i18n( "The target collection '%1' contains already\na collection with name '%2'.",
+                                           destCollection.name(), collection.name() ) );
+              return false;
+            }
+          }
         } else {
           const Item item = d->m_items.value( Item::fromUrl( url ).id() );
           if ( item.isValid() ) {
-            if ( item.parentCollection().id() == destCollection.id() ) {
+            if ( item.parentCollection().id() == destCollection.id() && action != Qt::CopyAction ) {
               kDebug() << "Error: source and destination of move are the same.";
               return false;
             }
@@ -481,7 +529,7 @@ bool EntityTreeModel::dropMimeData( const QMimeData * data, Qt::DropAction actio
       if ( !job )
         return false;
 
-      connect( job, SIGNAL( result( KJob* ) ), SLOT( pasteJobDone( KJob* ) ) );
+      connect( job, SIGNAL(result(KJob*)), SLOT(pasteJobDone(KJob*)) );
 
       // Accpet the event so that it doesn't propagate.
       return true;
@@ -537,6 +585,9 @@ QModelIndex EntityTreeModel::parent( const QModelIndex & index ) const
   if ( !index.isValid() )
     return QModelIndex();
 
+  if ( d->m_collectionFetchStrategy == InvisibleCollectionFetch || d->m_collectionFetchStrategy == FetchNoCollections )
+    return QModelIndex();
+
   const Node *node = reinterpret_cast<Node*>( index.internalPointer() );
 
   if ( !node )
@@ -554,7 +605,8 @@ QModelIndex EntityTreeModel::parent( const QModelIndex & index ) const
       return createIndex( 0, 0, reinterpret_cast<void *>( d->m_rootNode ) );
   }
 
-  const int row = d->indexOf( d->m_childEntities.value( collection.parentCollection().id() ), collection.id() );
+  Q_ASSERT( collection.parentCollection().isValid() );
+  const int row = d->indexOf<Node::Collection>( d->m_childEntities.value( collection.parentCollection().id() ), collection.id() );
 
   Q_ASSERT( row >= 0 );
   Node *parentNode = d->m_childEntities.value( collection.parentCollection().id() ).at( row );
@@ -566,30 +618,33 @@ int EntityTreeModel::rowCount( const QModelIndex & parent ) const
 {
   Q_D( const EntityTreeModel );
 
-  const Node *node = reinterpret_cast<Node*>( parent.internalPointer() );
+  if ( d->m_collectionFetchStrategy == InvisibleCollectionFetch || d->m_collectionFetchStrategy == FetchNoCollections ) {
+    if ( parent.isValid() )
+      return 0;
+    else
+      return d->m_items.size();
+  }
 
-  qint64 id;
   if ( !parent.isValid() ) {
     // If we're showing the root collection then it will be the only child of the root.
     if ( d->m_showRootCollection )
       return d->m_childEntities.value( -1 ).size();
-
-    id = d->m_rootCollection.id();
-  } else {
-
-    if ( !node )
-      return 0;
-
-    if ( Node::Item == node->type )
-      return 0;
-
-    id = node->id;
+    return d->m_childEntities.value( d->m_rootCollection.id() ).size();
   }
 
-  if ( parent.column() <= 0 )
-    return d->m_childEntities.value( id ).size();
+  if ( parent.column() != 0 )
+    return 0;
 
-  return 0;
+  const Node *node = reinterpret_cast<Node*>( parent.internalPointer() );
+
+  if ( !node )
+    return 0;
+
+  if ( Node::Item == node->type )
+    return 0;
+
+  Q_ASSERT( parent.isValid() );
+  return d->m_childEntities.value( node->id ).size();
 }
 
 int EntityTreeModel::entityColumnCount( HeaderGroup headerGroup ) const
@@ -606,8 +661,7 @@ QVariant EntityTreeModel::entityHeaderData( int section, Qt::Orientation orienta
   // Not needed in this model.
   Q_UNUSED( headerGroup );
 
-  if ( section == 0 && orientation == Qt::Horizontal && role == Qt::DisplayRole )
-  {
+  if ( section == 0 && orientation == Qt::Horizontal && role == Qt::DisplayRole ) {
     if ( d->m_rootCollection == Collection::root() )
       return i18nc( "@title:column Name of a thing", "Name" );
     return d->m_rootCollection.name();
@@ -630,7 +684,7 @@ QMimeData *EntityTreeModel::mimeData( const QModelIndexList &indexes ) const
 
   QMimeData *data = new QMimeData();
   KUrl::List urls;
-  foreach( const QModelIndex &index, indexes ) {
+  foreach ( const QModelIndex &index, indexes ) {
     if ( index.column() != 0 )
       continue;
 
@@ -640,7 +694,7 @@ QMimeData *EntityTreeModel::mimeData( const QModelIndexList &indexes ) const
     const Node *node = reinterpret_cast<Node*>( index.internalPointer() );
 
     if ( Node::Collection == node->type )
-      urls << d->m_collections.value( node->id ).url();
+      urls << d->m_collections.value( node->id ).url( Collection::UrlWithName );
     else if ( Node::Item == node->type )
       urls << d->m_items.value( node->id ).url( Item::UrlWithMimeType );
     else // if that happens something went horrible wrong
@@ -715,8 +769,8 @@ bool EntityTreeModel::setData( const QModelIndex &index, const QVariant &value,
         collection = value.value<Collection>();
 
       CollectionModifyJob *job = new CollectionModifyJob( collection, d->m_session );
-      connect( job, SIGNAL( result( KJob* ) ),
-               SLOT( updateJobDone( KJob* ) ) );
+      connect( job, SIGNAL(result(KJob*)),
+               SLOT(updateJobDone(KJob*)) );
 
       return false;
     } else if ( Node::Item == node->type ) {
@@ -751,8 +805,8 @@ bool EntityTreeModel::setData( const QModelIndex &index, const QVariant &value,
       }
 
       ItemModifyJob *itemModifyJob = new ItemModifyJob( item, d->m_session );
-      connect( itemModifyJob, SIGNAL( result( KJob* ) ),
-               SLOT( updateJobDone( KJob* ) ) );
+      connect( itemModifyJob, SIGNAL(result(KJob*)),
+               SLOT(updateJobDone(KJob*)) );
 
       return false;
     }
@@ -763,44 +817,18 @@ bool EntityTreeModel::setData( const QModelIndex &index, const QVariant &value,
 
 bool EntityTreeModel::canFetchMore( const QModelIndex & parent ) const
 {
-  Q_D( const EntityTreeModel );
-  const Item item = parent.data( ItemRole ).value<Item>();
-
-  if ( item.isValid() ) {
-    // items can't have more rows.
-    // TODO: Should I use this for fetching more of an item, ie more payload parts?
-    return false;
-  } else {
-    // but collections can...
-    const Collection::Id colId = parent.data( CollectionIdRole ).toULongLong();
-
-    // But the root collection can't...
-    if ( Collection::root().id() == colId )
-      return false;
-
-    foreach ( Node *node, d->m_childEntities.value( colId ) ) {
-      if ( Node::Item == node->type ) {
-        // Only try to fetch more from a collection if we don't already have items in it.
-        // Otherwise we'd spend all the time listing items in collections.
-        // This means that collections which don't contain items get a lot of item fetch jobs started on them.
-        // Will fix that later.
-        return false;
-      }
-    }
-
-    return true;
-  }
-
-  // TODO: It might be possible to get akonadi to tell us if a collection is empty
-  //       or not and use that information instead of assuming all collections are not empty.
-  //       Using Collection statistics?
+  Q_UNUSED(parent)
+  return false;
 }
 
 void EntityTreeModel::fetchMore( const QModelIndex & parent )
 {
   Q_D( EntityTreeModel );
 
-  if ( !canFetchMore( parent ) )
+  if ( !d->canFetchMore( parent ) )
+    return;
+
+  if ( d->m_collectionFetchStrategy == InvisibleCollectionFetch )
     return;
 
   if ( d->m_itemPopulation == ImmediatePopulation )
@@ -819,6 +847,10 @@ void EntityTreeModel::fetchMore( const QModelIndex & parent )
 bool EntityTreeModel::hasChildren( const QModelIndex &parent ) const
 {
   Q_D( const EntityTreeModel );
+
+  if ( d->m_collectionFetchStrategy == InvisibleCollectionFetch || d->m_collectionFetchStrategy == FetchNoCollections )
+    return parent.isValid() ? false : !d->m_items.isEmpty();
+
   // TODO: Empty collections right now will return true and get a little + to expand.
   // There is probably no way to tell if a collection
   // has child items in akonadi without first attempting an itemFetchJob...
@@ -851,9 +883,10 @@ QModelIndexList EntityTreeModel::match( const QModelIndex& start, int role, cons
     if ( role == CollectionRole ) {
       const Collection collection = value.value<Collection>();
       id = collection.id();
+    } else {
+      id = value.toLongLong();
     }
 
-    id = value.toLongLong();
     QModelIndexList list;
 
     const Collection collection = d->m_collections.value( id );
@@ -868,14 +901,14 @@ QModelIndexList EntityTreeModel::match( const QModelIndex& start, int role, cons
     return list;
   }
 
-  if ( role == ItemIdRole ) {
+  if ( role == ItemIdRole || role == ItemRole ) {
     Item::Id id;
-    if ( role == CollectionRole ) {
+    if ( role == ItemRole ) {
       const Item item = value.value<Item>();
       id = item.id();
+    } else {
+      id = value.toLongLong();
     }
-
-    id = value.toLongLong();
     QModelIndexList list;
 
     const Item item = d->m_items.value( id );
@@ -964,19 +997,19 @@ void EntityTreeModel::setItemPopulationStrategy( ItemPopulationStrategy strategy
   d->m_itemPopulation = strategy;
 
   if ( strategy == NoItemPopulation ) {
-    disconnect( d->m_monitor, SIGNAL( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ),
-            this, SLOT( monitoredItemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-    disconnect( d->m_monitor, SIGNAL( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ),
-            this, SLOT( monitoredItemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ) );
-    disconnect( d->m_monitor, SIGNAL( itemRemoved( const Akonadi::Item& ) ),
-            this, SLOT( monitoredItemRemoved( const Akonadi::Item& ) ) );
-    disconnect( d->m_monitor, SIGNAL( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
-            this, SLOT( monitoredItemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
+    disconnect( d->m_monitor, SIGNAL(itemAdded(Akonadi::Item,Akonadi::Collection)),
+            this, SLOT(monitoredItemAdded(Akonadi::Item,Akonadi::Collection)) );
+    disconnect( d->m_monitor, SIGNAL(itemChanged(Akonadi::Item,QSet<QByteArray>)),
+            this, SLOT(monitoredItemChanged(Akonadi::Item,QSet<QByteArray>)) );
+    disconnect( d->m_monitor, SIGNAL(itemRemoved(Akonadi::Item)),
+            this, SLOT(monitoredItemRemoved(Akonadi::Item)) );
+    disconnect( d->m_monitor, SIGNAL(itemMoved(Akonadi::Item,Akonadi::Collection,Akonadi::Collection)),
+            this, SLOT(monitoredItemMoved(Akonadi::Item,Akonadi::Collection,Akonadi::Collection)) );
 
-    disconnect( d->m_monitor, SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
-            this, SLOT( monitoredItemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
-    disconnect( d->m_monitor, SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
-            this, SLOT( monitoredItemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
+    disconnect( d->m_monitor, SIGNAL(itemLinked(Akonadi::Item,Akonadi::Collection)),
+            this, SLOT(monitoredItemLinked(Akonadi::Item,Akonadi::Collection)) );
+    disconnect( d->m_monitor, SIGNAL(itemUnlinked(Akonadi::Item,Akonadi::Collection)),
+            this, SLOT(monitoredItemUnlinked(Akonadi::Item,Akonadi::Collection)) );
   }
 
   d->m_monitor->d_ptr->useRefCounting = (strategy == LazyPopulation);
@@ -1024,18 +1057,20 @@ void EntityTreeModel::setCollectionFetchStrategy( CollectionFetchStrategy strate
   d->beginResetModel();
   d->m_collectionFetchStrategy = strategy;
 
-
-  if ( strategy == FetchNoCollections ) {
-    disconnect( d->m_monitor, SIGNAL( collectionChanged( const Akonadi::Collection& ) ),
-            this, SLOT( monitoredCollectionChanged( const Akonadi::Collection& ) ) );
-    disconnect( d->m_monitor, SIGNAL( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ),
-            this, SLOT( monitoredCollectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-    disconnect( d->m_monitor, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
-            this, SLOT( monitoredCollectionRemoved( const Akonadi::Collection& ) ) );
+  if ( strategy == FetchNoCollections || strategy == InvisibleCollectionFetch ) {
+    disconnect( d->m_monitor, SIGNAL(collectionChanged(Akonadi::Collection)),
+            this, SLOT(monitoredCollectionChanged(Akonadi::Collection)) );
+    disconnect( d->m_monitor, SIGNAL(collectionAdded(Akonadi::Collection,Akonadi::Collection)),
+            this, SLOT(monitoredCollectionAdded(Akonadi::Collection,Akonadi::Collection)) );
+    disconnect( d->m_monitor, SIGNAL(collectionRemoved(Akonadi::Collection)),
+            this, SLOT(monitoredCollectionRemoved(Akonadi::Collection)) );
     disconnect( d->m_monitor,
-            SIGNAL( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
-            this, SLOT( monitoredCollectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
-  }
+            SIGNAL(collectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)),
+            this, SLOT(monitoredCollectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)) );
+    d->m_monitor->fetchCollection( false );
+  } else
+    d->m_monitor->fetchCollection( true );
+
   d->endResetModel();
 }
 
@@ -1045,4 +1080,50 @@ EntityTreeModel::CollectionFetchStrategy EntityTreeModel::collectionFetchStrateg
   return d->m_collectionFetchStrategy;
 }
 
+static QPair<QList<const QAbstractProxyModel *>, const EntityTreeModel *> proxiesAndModel( const QAbstractItemModel *model )
+{
+  QList<const QAbstractProxyModel *> proxyChain;
+  const QAbstractProxyModel *proxy = qobject_cast<const QAbstractProxyModel *>( model );
+  const QAbstractItemModel *_model = model;
+  while ( proxy )
+  {
+    proxyChain.prepend( proxy );
+    _model = proxy->sourceModel();
+    proxy = qobject_cast<const QAbstractProxyModel *>( _model );
+  }
+
+  const EntityTreeModel *etm = qobject_cast<const EntityTreeModel *>( _model );
+  return qMakePair(proxyChain, etm);
+}
+
+static QModelIndex proxiedIndex( const QModelIndex &idx, QList<const QAbstractProxyModel *> proxyChain )
+{
+  QListIterator<const QAbstractProxyModel *> it( proxyChain );
+  QModelIndex _idx = idx;
+  while ( it.hasNext() )
+    _idx = it.next()->mapFromSource( _idx );
+  return _idx;
+}
+
+QModelIndex EntityTreeModel::modelIndexForCollection( const QAbstractItemModel *model, const Collection &collection )
+{
+  QPair<QList<const QAbstractProxyModel *>, const EntityTreeModel*> pair = proxiesAndModel( model );
+  QModelIndex idx = pair.second->d_ptr->indexForCollection( collection );
+  return proxiedIndex( idx, pair.first );
+}
+
+QModelIndexList EntityTreeModel::modelIndexesForItem( const QAbstractItemModel *model, const Item &item )
+{
+  QPair<QList<const QAbstractProxyModel *>, const EntityTreeModel*> pair = proxiesAndModel( model );
+  QModelIndexList list = pair.second->d_ptr->indexesForItem( item );
+  QModelIndexList proxyList;
+  foreach( const QModelIndex &idx, list )
+  {
+    const QModelIndex pIdx = proxiedIndex( idx, pair.first );
+    if ( pIdx.isValid() )
+      proxyList << pIdx;
+  }
+  return proxyList;
+}
+
 #include "entitytreemodel.moc"
