@@ -17,11 +17,14 @@
     02110-1301, USA.
 */
 
-#include "control.h"
 #include "itemappendtest.h"
-#include "collectionpathresolver_p.h"
-#include "testattribute.h"
 
+#include "control.h"
+#include "testattribute.h"
+#include "test_utils.h"
+
+#include <akonadi/agentinstance.h>
+#include <akonadi/agentmanager.h>
 #include <akonadi/attributefactory.h>
 #include <akonadi/collectionfetchjob.h>
 #include <akonadi/itemcreatejob.h>
@@ -33,27 +36,17 @@
 
 using namespace Akonadi;
 
-#include <qtest_kde.h>
-
-QTEST_KDEMAIN( ItemAppendTest, NoGUI )
-
-static Collection testFolder1;
+QTEST_AKONADIMAIN( ItemAppendTest, NoGUI )
 
 void ItemAppendTest::initTestCase()
 {
   Control::start();
 
-  // get the collections we run the tests on
-  CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive );
-  QVERIFY( job->exec() );
-  Collection::List list = job->collections();
-  Collection res2;
-  foreach ( const Collection &col, list )
-    if ( col.name() == "res2" )
-      res2 = col;
-  foreach ( const Collection &col, list )
-    if ( col.name() == "space folder" && col.parent() == res2.id() )
-      testFolder1 = col;
+  // switch all resources offline to reduce interference from them
+  foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() ) //krazy:exclude=foreach
+    agent.setIsOnline( false );
+
+  AttributeFactory::registerAttribute<TestAttribute>();
 }
 
 void ItemAppendTest::testItemAppend_data()
@@ -64,10 +57,14 @@ void ItemAppendTest::testItemAppend_data()
   QTest::newRow( "non empty" ) << QString( "remote-id" );
   QTest::newRow( "whitespace" ) << QString( "remote id" );
   QTest::newRow( "quotes" ) << QString ( "\"remote\" id" );
+  QTest::newRow( "brackets" ) << QString( "[remote id]" );
 }
 
 void ItemAppendTest::testItemAppend()
 {
+  const Collection testFolder1( collectionIdFromPath( "res2/space folder" ) );
+  QVERIFY( testFolder1.isValid() );
+
   QFETCH( QString, remoteId );
   Item ref; // for cleanup
 
@@ -76,25 +73,28 @@ void ItemAppendTest::testItemAppend()
   item.setMimeType( "application/octet-stream" );
   item.setFlag( "TestFlag" );
   item.setSize( 3456 );
-  ItemCreateJob *job = new ItemCreateJob( item, Collection( testFolder1 ), this );
-  QVERIFY( job->exec() );
+  ItemCreateJob *job = new ItemCreateJob( item, testFolder1, this );
+  AKVERIFYEXEC( job );
   ref = job->item();
+  QCOMPARE( ref.parentCollection(), testFolder1 );
 
   ItemFetchJob *fjob = new ItemFetchJob( testFolder1, this );
-  QVERIFY( fjob->exec() );
+  fjob->fetchScope().setAncestorRetrieval( ItemFetchScope::Parent );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   QCOMPARE( fjob->items()[0], ref );
   QCOMPARE( fjob->items()[0].remoteId(), remoteId );
   QVERIFY( fjob->items()[0].flags().contains( "TestFlag" ) );
+  QCOMPARE( fjob->items()[0].parentCollection(), ref.parentCollection() );
 
   qint64 size = 3456;
   QCOMPARE( fjob->items()[0].size(), size );
 
   ItemDeleteJob *djob = new ItemDeleteJob( ref, this );
-  QVERIFY( djob->exec() );
+  AKVERIFYEXEC( djob );
 
   fjob = new ItemFetchJob( testFolder1, this );
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QVERIFY( fjob->items().isEmpty() );
 }
 
@@ -102,37 +102,71 @@ void ItemAppendTest::testContent_data()
 {
   QTest::addColumn<QByteArray>( "data" );
 
-  QTest::newRow( "empty" ) << QByteArray();
+  QTest::newRow( "null" ) << QByteArray();
+  QTest::newRow( "empty" ) << QByteArray( "" );
+  QTest::newRow( "nullbyte" ) << QByteArray( "\0", 1 );
+  QTest::newRow( "nullbyte2" ) << QByteArray( "\0X", 2 );
   QString utf8string = QString::fromUtf8("äöüß@€µøđ¢©®");
   QTest::newRow( "utf8" ) << utf8string.toUtf8();
   QTest::newRow( "newlines" ) << QByteArray("\nsome\n\nbreaked\ncontent\n\n");
+  QByteArray b;
+  QTest::newRow( "big" ) << b.fill( 'a', 1 << 20 );
+  QTest::newRow( "bignull" ) << b.fill( '\0', 1 << 20 );
+  QTest::newRow( "bigcr" ) << b.fill( '\r', 1 << 20 );
+  QTest::newRow( "biglf" ) << b.fill( '\n', 1 << 20 );
 }
 
 void ItemAppendTest::testContent()
 {
+  const Collection testFolder1( collectionIdFromPath( "res2/space folder" ) );
+  QVERIFY( testFolder1.isValid() );
+
   QFETCH( QByteArray, data );
 
   Item item;
   item.setMimeType( "application/octet-stream" );
   item.setPayload( data );
 
-  ItemCreateJob* job = new ItemCreateJob( item, Collection( testFolder1 ), this );
-  QVERIFY( job->exec() );
+  ItemCreateJob* job = new ItemCreateJob( item, testFolder1, this );
+  AKVERIFYEXEC( job );
   Item ref = job->item();
 
   ItemFetchJob *fjob = new ItemFetchJob( testFolder1, this );
+  fjob->fetchScope().setCacheOnly( true );
   fjob->fetchScope().fetchFullPayload();
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   Item item2 = fjob->items().first();
-  QCOMPARE( data, item2.payload<QByteArray>() );
+  QCOMPARE( item2.payload<QByteArray>(), data );
 
   ItemDeleteJob *djob = new ItemDeleteJob( ref, this );
-  QVERIFY( djob->exec() );
+  AKVERIFYEXEC( djob );
+}
+
+void ItemAppendTest::testNewMimetype()
+{
+  const Collection col( collectionIdFromPath( "res2/space folder" ) );
+  QVERIFY( col.isValid() );
+
+  Item item;
+  item.setMimeType( "application/new-type" );
+  ItemCreateJob *job = new ItemCreateJob( item, col, this );
+  AKVERIFYEXEC( job );
+
+  item = job->item();
+  QVERIFY( item.isValid() );
+
+  ItemFetchJob *fetch = new ItemFetchJob( item, this );
+  AKVERIFYEXEC( fetch );
+  QCOMPARE( fetch->items().count(), 1 );
+  QCOMPARE( fetch->items().first().mimeType(), item.mimeType() );
 }
 
 void ItemAppendTest::testIllegalAppend()
 {
+  const Collection testFolder1( collectionIdFromPath( "res2/space folder" ) );
+  QVERIFY( testFolder1.isValid() );
+
   Item item;
   item.setMimeType( "application/octet-stream" );
 
@@ -140,16 +174,9 @@ void ItemAppendTest::testIllegalAppend()
   ItemCreateJob *job = new ItemCreateJob( item, Collection( INT_MAX ), this );
   QVERIFY( !job->exec() );
 
-  // adding item with non-existing mimetype
-  Item item2;
-  item2.setMimeType( "wrong/type" );
-  job = new ItemCreateJob( item2, Collection( testFolder1 ), this );
-  QVERIFY( !job->exec() );
-
   // adding item into a collection which can't handle items of this type
-  CollectionPathResolver *resolver = new CollectionPathResolver( "res1/foo/bla", this );
-  QVERIFY( resolver->exec() );
-  const Collection col = Collection( resolver->collection() );
+  const Collection col( collectionIdFromPath( "res1/foo/bla" ) );
+  QVERIFY( col.isValid() );
   job = new ItemCreateJob( item, col, this );
   QEXPECT_FAIL( "", "Test not yet implemented in the server.", Continue );
   QVERIFY( !job->exec() );
@@ -157,21 +184,22 @@ void ItemAppendTest::testIllegalAppend()
 
 void ItemAppendTest::testMultipartAppend()
 {
-  AttributeFactory::registerAttribute<TestAttribute>();
+  const Collection testFolder1( collectionIdFromPath( "res2/space folder" ) );
+  QVERIFY( testFolder1.isValid() );
 
   Item item;
   item.setMimeType( "application/octet-stream" );
   item.setPayload<QByteArray>( "body data" );
   item.attribute<TestAttribute>( Item::AddIfMissing )->data = "extra data";
   item.setFlag( "TestFlag" );
-  ItemCreateJob *job = new ItemCreateJob( item, Collection( testFolder1 ), this );
-  QVERIFY( job->exec() );
+  ItemCreateJob *job = new ItemCreateJob( item, testFolder1, this );
+  AKVERIFYEXEC( job );
   Item ref = job->item();
 
   ItemFetchJob *fjob = new ItemFetchJob( ref, this );
   fjob->fetchScope().fetchFullPayload();
   fjob->fetchScope().fetchAttribute<TestAttribute>();
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items().first();
   QCOMPARE( item.payload<QByteArray>(), QByteArray( "body data" ) );
@@ -180,7 +208,60 @@ void ItemAppendTest::testMultipartAppend()
   QVERIFY( item.flags().contains( "TestFlag" ) );
 
   ItemDeleteJob *djob = new ItemDeleteJob( ref, this );
-  QVERIFY( djob->exec() );
+  AKVERIFYEXEC( djob );
+}
+
+void ItemAppendTest::testInvalidMultipartAppend()
+{
+  Item item;
+  item.setMimeType( "application/octet-stream" );
+  item.setPayload<QByteArray>( "body data" );
+  item.attribute<TestAttribute>( Item::AddIfMissing )->data = "extra data";
+  item.setFlag( "TestFlag" );
+  ItemCreateJob *job = new ItemCreateJob( item, Collection( -1 ), this );
+  QVERIFY( !job->exec() );
+
+  Item item2;
+  item2.setMimeType( "application/octet-stream" );
+  item2.setPayload<QByteArray>( "more body data" );
+  item2.attribute<TestAttribute>( Item::AddIfMissing )->data = "even more extra data";
+  item2.setFlag( "TestFlag" );
+  ItemCreateJob *job2 = new ItemCreateJob( item2, Collection( -1 ), this );
+  QVERIFY( !job2->exec() );
+}
+
+void ItemAppendTest::testItemSize_data()
+{
+  QTest::addColumn<Akonadi::Item>( "item" );
+  QTest::addColumn<qint64>( "size" );
+
+  Item i( "application/octet-stream" );
+  i.setPayload( QByteArray( "ABCD" ) );
+
+  QTest::newRow( "auto size" ) << i << 4ll;
+  i.setSize( 3 );
+  QTest::newRow( "too small" ) << i << 4ll;
+  i.setSize( 10 );
+  QTest::newRow( "too large" ) << i << 10ll;
+}
+
+void ItemAppendTest::testItemSize()
+{
+  QFETCH( Akonadi::Item, item );
+  QFETCH( qint64, size );
+
+  const Collection col( collectionIdFromPath( "res2/space folder" ) );
+  QVERIFY( col.isValid() );
+
+  ItemCreateJob *create = new ItemCreateJob( item, col, this );
+  AKVERIFYEXEC( create );
+  Item newItem = create->item();
+
+  ItemFetchJob *fetch = new ItemFetchJob( newItem, this );
+  AKVERIFYEXEC( fetch );
+  QCOMPARE( fetch->items().count(), 1 );
+
+  QCOMPARE( fetch->items().first().size(), size );
 }
 
 #include "itemappendtest.moc"
