@@ -22,7 +22,9 @@
 
 #include "scheduler.h"
 #include "calendar.h"
+#ifndef KDEPIM_NO_KRESOURCES
 #include "calendarresources.h"
+#endif
 #include "event.h"
 #include "todo.h"
 #include "freebusy.h"
@@ -84,15 +86,19 @@ QString ScheduleMessage::statusName( ScheduleMessage::Status status )
 {
   switch( status ) {
   case PublishNew:
-    return i18nc( "@item new message posting", "New Message Publish" );
+    return i18nc( "@item this is a new scheduling message",
+                  "New Scheduling Message" );
   case PublishUpdate:
-    return i18nc( "@item updated message", "Updated Message Published" );
+    return i18nc( "@item this is an update to an existing scheduling message",
+                  "Updated Scheduling Message" );
   case Obsolete:
     return i18nc( "@item obsolete status", "Obsolete" );
   case RequestNew:
-    return i18nc( "@item request new message posting", "Request New Message" );
+    return i18nc( "@item this is a request for a new scheduling message",
+                  "New Scheduling Message Request" );
   case RequestUpdate:
-    return i18nc( "@item request updated posting", "Request Updated Message" );
+    return i18nc( "@item this is a request for an update to an existing scheduling message",
+                  "Updated Scheduling Message Request" );
   default:
     return i18nc( "@item unknown status", "Unknown Status: %1", int( status ) );
   }
@@ -345,8 +351,8 @@ bool Scheduler::acceptRequest( IncidenceBase *incidence,
           kError() << "assigning different incidence types";
           res = false;
         } else {
-          i->setSchedulingID( inc->uid() );
           i->setUid( oldUid );
+          i->setSchedulingID( inc->uid() );
         }
         deleteTransaction( incidence );
         return res;
