@@ -69,11 +69,13 @@ namespace KIMAP
       bool startAuthentication();
       bool answerChallenge(const QByteArray &data);
       void sslResponse(bool response);
+      void saveServerGreeting(const Message &response);
 
       LoginJob *q;
 
       QString userName;
       QString password;
+      QString serverGreeting;
 
       LoginJob::EncryptionMode encryptionMode;
       QString authMode;
@@ -167,29 +169,77 @@ void LoginJob::setPassword( const QString &password )
 void LoginJob::doStart()
 {
   Q_D(LoginJob);
-  if (d->encryptionMode == SslV2 || d->encryptionMode == SslV3 || d->encryptionMode == SslV3_1 || d->encryptionMode == AnySslVersion) {
+
+  // Don't authenticate on a session in the authenticated state
+  if ( session()->state() == Session::Authenticated || session()->state() == Session::Selected ) {
+    setError( UserDefinedError );
+    setErrorText( i18n("IMAP session in the wrong state for authentication") );
+    emitResult();
+    return;
+  }
+
+  // Trigger encryption negotiation only if needed
+  EncryptionMode encryptionMode = d->encryptionMode;
+
+  switch ( d->sessionInternal()->negotiatedEncryption() ) {
+  case KTcpSocket::UnknownSslVersion:
+    break; // Do nothing the encryption mode still needs to be negotiated
+
+  // For the other cases, pretend we're going unencrypted as that's the
+  // encryption mode already set on the session
+  // (so for instance we won't issue another STARTTLS for nothing if that's
+  // not needed)
+  case KTcpSocket::SslV2:
+    if ( encryptionMode==SslV2 ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  case KTcpSocket::SslV3:
+    if ( encryptionMode==SslV3 ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  case KTcpSocket::TlsV1:
+    if ( encryptionMode==TlsV1 ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  case KTcpSocket::AnySslVersion:
+    if ( encryptionMode==AnySslVersion ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  }
+
+  if (encryptionMode == SslV2
+   || encryptionMode == SslV3
+   || encryptionMode == SslV3_1
+   || encryptionMode == AnySslVersion) {
     KTcpSocket::SslVersion version = KTcpSocket::SslV2;
-    if (d->encryptionMode == SslV3)
+    if (encryptionMode == SslV3)
       version = KTcpSocket::SslV3;
-    if (d->encryptionMode == SslV3_1)
+    if (encryptionMode == SslV3_1)
       version = KTcpSocket::SslV3_1;
-    if (d->encryptionMode == AnySslVersion)
+    if (encryptionMode == AnySslVersion)
       version = KTcpSocket::AnySslVersion;
     d->sessionInternal()->startSsl(version);
-  } else  if (d->encryptionMode == Unencrypted ) {
+
+  } else if (encryptionMode == TlsV1) {
+    d->authState = LoginJobPrivate::StartTls;
+    d->tags << d->sessionInternal()->sendCommand( "STARTTLS" );
+
+  } else  if (encryptionMode == Unencrypted ) {
     if (d->authMode.isEmpty()) {
+      d->authState = LoginJobPrivate::Login;
       d->tags << d->sessionInternal()->sendCommand( "LOGIN",
-                                                  quoteIMAP( d->userName ).toUtf8()
+                                                  '"'+quoteIMAP( d->userName ).toUtf8()+'"'
                                                  +' '
-                                                 +quoteIMAP(d->password ).toUtf8() );
+                                                 +'"'+quoteIMAP(d->password ).toUtf8()+'"' );
     } else {
       if (!d->startAuthentication()) {
         emitResult();
       }
     }
-  } else if (d->encryptionMode == TlsV1) {
-    d->authState = LoginJobPrivate::StartTls;
-    d->tags << d->sessionInternal()->sendCommand( "STARTTLS" );
   }
 }
 
@@ -197,6 +247,9 @@ void LoginJob::handleResponse( const Message &response )
 {
   Q_D(LoginJob);
 
+  if ( response.content.isEmpty() )
+    return;
+
   //set the actual command name for standard responses
   QString commandName = i18n("Login");
   if (d->authState == LoginJobPrivate::Capability) {
@@ -205,83 +258,149 @@ void LoginJob::handleResponse( const Message &response )
     commandName = i18n("StartTls");
   }
 
-  if ( !response.content.isEmpty()
-       && d->tags.contains( response.content.first().toString() ) ) {
-    if ( response.content.size() < 2 ) {
-      setErrorText( i18n("%1 failed, malformed reply from the server.", commandName) );
+  enum ResponseCode {
+    OK,
+    ERR,
+    UNTAGGED,
+    CONTINUATION,
+    MALFORMED
+  };
+
+  QByteArray tag = response.content.first().toString();
+  ResponseCode code;
+
+  if ( tag == "+" ) {
+    code = CONTINUATION;
+  } else if ( tag == "*" ) {
+    if ( response.content.size() < 2 )
+      code = MALFORMED; // Received empty untagged response
+    else
+      code = UNTAGGED;
+  } else if ( d->tags.contains(tag) ) {
+    if ( response.content.size() < 2 )
+      code = MALFORMED;
+    else if ( response.content[1].toString() == "OK" )
+      code = OK;
+    else
+      code = ERR;
+  }
+
+  switch (code) {
+    case MALFORMED:
+      // We'll handle it later
+      break;
+
+    case ERR:
+      //server replied with NO or BAD for SASL authentication
+      if (d->authState == LoginJobPrivate::Authenticate)
+        sasl_dispose( &d->conn );
+
+      setError( UserDefinedError );
+      setErrorText( i18n("%1 failed, server replied: %2", commandName, response.toString().constData()) );
       emitResult();
-    } else if ( response.content[1].toString() != "OK" ) {
-        //server replied with NO or BAD for SASL authentication
-        if (d->authState == LoginJobPrivate::Authenticate) {
-          sasl_dispose( &d->conn );
+      return;
+
+    case UNTAGGED:
+      // The only untagged response interesting for us here is CAPABILITY
+      if ( response.content[1].toString() == "CAPABILITY" ) {
+        QList<Message::Part>::const_iterator p = response.content.begin() + 2;
+        while (p != response.content.end()) {
+          QString capability = p->toString();
+          d->capabilities << capability;
+          if (capability == "LOGINDISABLED")
+            d->plainLoginDisabled = true;
+          ++p;
         }
+        kDebug() << "Capabilities updated: " << d->capabilities;
+      }
+      break;
 
-        setError( UserDefinedError );
-        setErrorText( i18n("%1 failed, server replied: %2", commandName, response.toString().constData()) );
-        emitResult();
-    } else if ( response.content[1].toString() == "OK")    {
-      if (d->authState == LoginJobPrivate::Authenticate) {
-        sasl_dispose( &d->conn ); //SASL authentication done
-        emitResult();
-      } else if (d->authState == LoginJobPrivate::Capability) {
+    case CONTINUATION:
+      if (d->authState != LoginJobPrivate::Authenticate) {
+        // Received unexpected continuation response for something
+        // other than AUTHENTICATE command
+        code = MALFORMED;
+        break;
+      }
 
-        //cleartext login, if enabled
-        if (d->authMode.isEmpty()) {
-          if (d->plainLoginDisabled) {
-            setError( UserDefinedError );
-            setErrorText( i18n("Login failed, plain login is disabled by the server.") );
-            emitResult();
+      if ( d->authMode == QLatin1String( "PLAIN" ) ) {
+        if ( response.content.size()>1 && response.content.at( 1 ).toString()=="OK" ) {
+          return;
+        }
+        QByteArray challengeResponse;
+        challengeResponse+= '\0';
+        challengeResponse+= d->userName.toUtf8();
+        challengeResponse+= '\0';
+        challengeResponse+= d->password.toUtf8();
+        challengeResponse = challengeResponse.toBase64();
+        d->sessionInternal()->sendData( challengeResponse );
+      } else if ( response.content.size() >= 2 ) {
+        if (!d->answerChallenge(QByteArray::fromBase64(response.content[1].toString()))) {
+          emitResult(); //error, we're done
+        }
+      } else {
+        // Received empty continuation for authMode other than PLAIN
+        code = MALFORMED;
+      }
+      break;
+
+    case OK:
+
+      switch (d->authState) {
+        case LoginJobPrivate::StartTls:
+          d->sessionInternal()->startSsl(KTcpSocket::TlsV1);
+          break;
+
+        case LoginJobPrivate::Capability:
+          //cleartext login, if enabled
+          if (d->authMode.isEmpty()) {
+            if (d->plainLoginDisabled) {
+              setError( UserDefinedError );
+              setErrorText( i18n("Login failed, plain login is disabled by the server.") );
+              emitResult();
+            } else {
+              d->authState = LoginJobPrivate::Login;
+              d->tags << d->sessionInternal()->sendCommand( "LOGIN",
+                                                          '"'+quoteIMAP( d->userName ).toUtf8()+'"'
+                                                         +' '
+                                                         +'"'+quoteIMAP( d->password ).toUtf8()+'"');
+            }
           } else {
-            d->authState = LoginJobPrivate::Login;
-            d->tags << d->sessionInternal()->sendCommand( "LOGIN",
-                                                        quoteIMAP( d->userName ).toUtf8()
-                                                       +' '
-                                                       +quoteIMAP( d->password ).toUtf8() );
-          }
-        }
-
-        //find the selected SASL authentication method
-        Q_FOREACH(const QString &capability, d->capabilities) {
-          if (capability.startsWith(QLatin1String("AUTH="))) {
-            QString authType = capability.mid(5);
-            if (authType == d->authMode) {
-                if (!d->startAuthentication()) {
-                  emitResult(); //problem, we're done
+            bool authModeSupported = false;
+            //find the selected SASL authentication method
+            Q_FOREACH(const QString &capability, d->capabilities) {
+              if (capability.startsWith(QLatin1String("AUTH="))) {
+                if (capability.mid(5) == d->authMode) {
+                  authModeSupported = true;
+                  break;
                 }
+              }
+            }
+            if (!authModeSupported) {
+              setError( UserDefinedError );
+              setErrorText( i18n("Login failed, authentication mode %1 is not supported by the server.", d->authMode) );
+              emitResult();
+            } else if (!d->startAuthentication()) {
+              emitResult(); //problem, we're done
             }
           }
-        }
-      } else if (d->authState == LoginJobPrivate::StartTls) {
-        d->sessionInternal()->startSsl(KTcpSocket::TlsV1);
-      } else {
-        emitResult(); //got an OK, command done
+          break;
+
+        case LoginJobPrivate::Authenticate:
+          sasl_dispose( &d->conn ); //SASL authentication done
+          // Fall through
+        case LoginJobPrivate::Login:
+          d->saveServerGreeting( response );
+          emitResult(); //got an OK, command done
+          break;
+
       }
-    }
-  } else if ( response.content.size() >= 2 ) {
-    if ( d->authState == LoginJobPrivate::Authenticate ) {
-      if (!d->answerChallenge(response.content[1].toString())) {
-        emitResult(); //error, we're done
-      }
-    } else if ( response.content[1].toString()=="CAPABILITY" ) {
-      bool authModeSupported = d->authMode.isEmpty();
-      for (int i = 2; i < response.content.size(); ++i) {
-        QString capability = response.content[i].toString();
-        d->capabilities << capability;
-        if (capability == "LOGINDISABLED") {
-          d->plainLoginDisabled = true;
-        }
-        QString authMode = capability.mid(5);
-        if (authMode == d->authMode) {
-          authModeSupported = true;
-        }
-      }
-      kDebug() << "Capabilities after STARTTLS: " << d->capabilities;
-      if (!authModeSupported) {
-        setError( UserDefinedError );
-        setErrorText( i18n("Login failed, authentication mode %1 is not supported by the server.", d->authMode) );
-        d->authState = LoginJobPrivate::Login; //just to treat the upcoming OK correctly
-      }
-    }
+
+  }
+
+  if ( code == MALFORMED ) {
+    setErrorText( i18n("%1 failed, malformed reply from the server.", commandName) );
+    emitResult();
   }
 }
 
@@ -330,7 +449,11 @@ bool LoginJobPrivate::startAuthentication()
   QByteArray tmp = QByteArray::fromRawData( out, outlen );
   QByteArray challenge = tmp.toBase64();
 
-  tags << sessionInternal()->sendCommand( "AUTHENTICATE", authMode.toLatin1() + ' ' + challenge );
+  if ( challenge.isEmpty() ) {
+    tags << sessionInternal()->sendCommand( "AUTHENTICATE", authMode.toLatin1() );
+  } else {
+    tags << sessionInternal()->sendCommand( "AUTHENTICATE", authMode.toLatin1() + ' ' + challenge );
+  }
 
   return true;
 }
@@ -428,10 +551,37 @@ void LoginJob::connectionLost()
   //don't emit the result if the connection was lost before getting the tls result, as it can mean
   //the TLS handshake failed and the socket was reconnected in normal mode
   if (d->authState != LoginJobPrivate::StartTls) {
+    setError( ERR_COULD_NOT_CONNECT );
+    setErrorText( i18n("Connection to server lost.") );
     emitResult();
   }
 
 }
 
+void LoginJobPrivate::saveServerGreeting(const Message &response)
+{
+  // Concatenate the parts of the server response into a string, while dropping the first two parts
+  // (the response tag and the "OK" code), and being careful not to add useless extra whitespace.
+
+  for ( int i=2; i<response.content.size(); i++) {
+    if ( response.content.at(i).type()==Message::Part::List ) {
+      serverGreeting+='(';
+      foreach ( const QByteArray &item, response.content.at(i).toList() ) {
+        serverGreeting+=item+' ';
+      }
+      serverGreeting.chop(1);
+      serverGreeting+=") ";
+    } else {
+      serverGreeting+=response.content.at(i).toString()+' ';
+    }
+  }
+  serverGreeting.chop(1);
+}
+
+QString LoginJob::serverGreeting() const
+{
+  Q_D(const LoginJob);
+  return d->serverGreeting;
+}
 
 #include "loginjob.moc"
