@@ -82,7 +82,7 @@ InteractiveSMTPServerWindow::~InteractiveSMTPServerWindow()
     if ( mSocket ) {
         mSocket->close();
         if ( mSocket->state() == QAbstractSocket::ClosingState )
-            connect( mSocket, SIGNAL(delayedCloseFinished()),
+            connect( mSocket, SIGNAL(disconnected()),
                      mSocket, SLOT(deleteLater()) );
         else
             mSocket->deleteLater();
@@ -105,7 +105,7 @@ InteractiveSMTPServer::InteractiveSMTPServer( QObject* parent )
   listen( localhost, 2525 );
   setMaxPendingConnections( 1 );
 
-  connect( this, SIGNAL( newConnection() ), this, SLOT( newConnectionAvailable() ) );
+  connect( this, SIGNAL(newConnection()), this, SLOT(newConnectionAvailable()) );
 }
 
 void InteractiveSMTPServer::newConnectionAvailable()
