@@ -36,6 +36,8 @@
 
 #include "kmime_content.h"
 #include "kmime_content_p.h"
+#include "kmime_codecs.h"
+#include "kmime_message.h"
 #include "kmime_header_parsing.h"
 #include "kmime_header_parsing_p.h"
 #include "kmime_parsers.h"
@@ -96,7 +98,7 @@ Content::~Content()
 
 bool Content::hasContent() const
 {
-  return !d_ptr->head.isEmpty() || !d_ptr->body.isEmpty() || !d_ptr->contents.isEmpty();
+  return !d_ptr->head.isEmpty() || !d_ptr->body.isEmpty() || !d_ptr->contents().isEmpty();
 }
 
 void Content::setContent( const QList<QByteArray> &l )
@@ -131,22 +133,7 @@ void Content::setContent( const QList<QByteArray> &l )
 void Content::setContent( const QByteArray &s )
 {
   Q_D(Content);
-  d->head.clear();
-  d->body.clear();
-
-  // empty header
-  if ( s.startsWith( '\n' ) ) {
-    d->body = s.right( s.length() - 1 );
-    return;
-  }
-
-  int pos = s.indexOf( "\n\n", 0 );
-  if ( pos > -1 ) {
-    d->head = s.left( ++pos );  //header *must* end with "\n" !!
-    d->body = s.mid( pos + 1, s.length() - pos - 1 );
-  } else {
-    d->head = s;
-  }
+  KMime::HeaderParsing::extractHeaderAndBody( s, d->head, d->body );
 }
 
 QByteArray Content::head() const
@@ -177,6 +164,7 @@ void Content::parse()
 
   // Clean up old headers and parse them again.
   qDeleteAll( h_eaders );
+  h_eaders.clear();
   h_eaders = HeaderParsing::parseHeaders( d->head );
   foreach( Headers::Base *h, h_eaders ) {
     h->setParent( this );
@@ -189,8 +177,9 @@ void Content::parse()
   }
 
   // Clean up old sub-Contents and parse them again.
-  qDeleteAll( d->contents );
-  d->contents.clear();
+  qDeleteAll( d->multipartContents );
+  d->multipartContents.clear();
+  d->clearBodyMessage();
   Headers::ContentType *ct = contentType();
   if( ct->isText() ) {
     // This content is either text, or of unknown type.
@@ -209,12 +198,23 @@ void Content::parse()
       // This is actual MIME multipart content.
     } else {
       // Parsing failed; treat this content as "text/plain".
-      kWarning() << "Failed to parse multipart. Treating as text/plain.";
       ct->setMimeType( "text/plain" );
       ct->setCharset( "US-ASCII" );
     }
   } else {
-    // This content is something else, like an image or something...
+    // This content is something else, like an encapsulated message or a binary attachment
+    // or something like that
+    if ( bodyIsMessage() ) {
+      d->bodyAsMessage = Message::Ptr( new Message );
+      d->bodyAsMessage->setContent( d->body );
+      d->bodyAsMessage->setFrozen( d->frozen );
+      d->bodyAsMessage->parse();
+      d->bodyAsMessage->d_ptr->parent = this;
+
+      // Clear the body, as it is now represented by d->bodyAsMessage. This is the same behavior
+      // as with multipart contents, since parseMultipart() clears the body as well
+      d->body.clear();
+    }
   }
 }
 
@@ -267,9 +267,10 @@ void Content::clearContents( bool del )
 {
   Q_D(Content);
   if( del ) {
-    qDeleteAll( d->contents );
+    qDeleteAll( d->multipartContents );
   }
-  d->contents.clear();
+  d->multipartContents.clear();
+  d->clearBodyMessage();
 }
 
 QByteArray Content::encodedContent( bool useCrLf )
@@ -280,7 +281,19 @@ QByteArray Content::encodedContent( bool useCrLf )
   // Head.
   e = d->head;
   e += '\n';
+  e += encodedBody();
 
+  if ( useCrLf ) {
+    return LFtoCRLF( e );
+  } else {
+    return e;
+  }
+}
+
+QByteArray Content::encodedBody()
+{
+  Q_D( Content );
+  QByteArray e;
   // Body.
   if( d->frozen ) {
     // This Content is frozen.
@@ -291,13 +304,16 @@ QByteArray Content::encodedContent( bool useCrLf )
       // Use the body as it was before parsing.
       e += d->frozenBody;
     }
+  } else if( bodyIsMessage() && d->bodyAsMessage ) {
+    // This is an encapsulated message
+    // No encoding needed, as the ContentTransferEncoding can only be 7bit
+    // for encapsulated messages
+    e += d->bodyAsMessage->encodedContent();
   } else if( !d->body.isEmpty() ) {
     // This is a single-part Content.
     Headers::ContentTransferEncoding *enc = contentTransferEncoding();
-    Q_ASSERT( enc->encoding() != Headers::CEuuenc );
-    Q_ASSERT( enc->encoding() != Headers::CEbinary );
 
-    if (enc->needToEncode()) {
+    if ( enc->needToEncode() ) {
       if ( enc->encoding() == Headers::CEquPr ) {
         e += KCodecs::quotedPrintableEncode( d->body, false );
       } else {
@@ -307,63 +323,70 @@ QByteArray Content::encodedContent( bool useCrLf )
     } else {
       e += d->body;
     }
-  } else if ( !d->contents.isEmpty() ) {
+  }
+
+  if ( !d->frozen && !d->multipartContents.isEmpty() ) {
     // This is a multipart Content.
     Headers::ContentType *ct=contentType();
     QByteArray boundary = "\n--" + ct->boundary();
 
+    if ( !d->preamble.isEmpty() )
+      e += d->preamble;
+
     //add all (encoded) contents separated by boundaries
-    foreach ( Content *c, d->contents ) {
+    foreach ( Content *c, d->multipartContents ) {
       e+=boundary + '\n';
       e += c->encodedContent( false );  // don't convert LFs here, we do that later!!!!!
     }
     //finally append the closing boundary
     e += boundary+"--\n";
-  };
 
-  if ( useCrLf ) {
-    return LFtoCRLF( e );
-  } else {
-    return e;
-  }
+    if ( !d->epilogue.isEmpty() )
+      e += d->epilogue;
+  };
+  return e;
 }
 
 QByteArray Content::decodedContent()
 {
-  QByteArray temp, ret;
+  QByteArray ret;
   Headers::ContentTransferEncoding *ec=contentTransferEncoding();
   bool removeTrailingNewline=false;
-  int size = d_ptr->body.length();
 
-  if ( size == 0 ) {
+  if ( d_ptr->body.length() == 0 ) {
     return ret;
   }
 
-  temp.resize( size );
-  memcpy( temp.data(), d_ptr->body.data(), size );
-
   if ( ec->decoded() ) {
-    ret = temp;
+    ret = d_ptr->body;
     removeTrailingNewline = true;
   } else {
     switch( ec->encoding() ) {
     case Headers::CEbase64 :
-      KCodecs::base64Decode( temp, ret );
+    {
+      KMime::Codec *codec = KMime::Codec::codecForName( "base64" );
+      Q_ASSERT( codec );
+      ret.resize( codec->maxDecodedSizeFor( d_ptr->body.size() ) );
+      KMime::Decoder* decoder = codec->makeDecoder();
+      QByteArray::const_iterator inputIt = d_ptr->body.constBegin();
+      QByteArray::iterator resultIt = ret.begin();
+      decoder->decode( inputIt, d_ptr->body.constEnd(), resultIt, ret.end() );
+      ret.truncate( resultIt - ret.begin() );
       break;
+    }
     case Headers::CEquPr :
       ret = KCodecs::quotedPrintableDecode( d_ptr->body );
-      ret.resize( ret.size() - 1 );  // remove null-char
       removeTrailingNewline = true;
       break;
     case Headers::CEuuenc :
-      KCodecs::uudecode( temp, ret );
+      KCodecs::uudecode( d_ptr->body, ret );
       break;
     case Headers::CEbinary :
-      ret = temp;
+      ret = d_ptr->body;
       removeTrailingNewline = false;
       break;
     default :
-      ret = temp;
+      ret = d_ptr->body;
       removeTrailingNewline = true;
     }
   }
@@ -383,7 +406,12 @@ QString Content::decodedText( bool trimText, bool removeTrailingNewlines )
 
   bool ok = true;
   QTextCodec *codec =
-    KGlobal::charsets()->codecForName( contentType()->charset(), ok );
+    KGlobal::charsets()->codecForName( QLatin1String( contentType()->charset() ), ok );
+  if ( !ok  || codec == NULL ) { // no suitable codec found => try local settings and hope the best ;-)
+    codec = KGlobal::locale()->codecForEncoding();
+    QByteArray chset = KGlobal::locale()->encoding();
+    contentType()->setCharset( chset );
+  }
 
   QString s = codec->toUnicode( d_ptr->body.data(), d_ptr->body.length() );
 
@@ -396,14 +424,14 @@ QString Content::decodedText( bool trimText, bool removeTrailingNewlines )
         }
       }
       else {
-        if ( s[i] != '\n' ) {
+        if ( s[i] != QLatin1Char( '\n' ) ) {
           break;
         }
       }
     }
     s.truncate( i + 1 );
   } else {
-    if ( s.right( 1 ) == "\n" ) {
+    if ( s.right( 1 ) == QLatin1String( "\n" ) ) {
       s.truncate( s.length() - 1 ); // remove trailing new-line
     }
   }
@@ -415,7 +443,7 @@ void Content::fromUnicodeString( const QString &s )
 {
   bool ok = true;
   QTextCodec *codec =
-    KGlobal::charsets()->codecForName( contentType()->charset(), ok );
+    KGlobal::charsets()->codecForName( QLatin1String( contentType()->charset() ), ok );
 
   if ( !ok ) { // no suitable codec found => try local settings and hope the best ;-)
     codec = KGlobal::locale()->codecForEncoding();
@@ -435,7 +463,7 @@ Content *Content::textContent()
   if ( contentType()->isText() ) {
     ret = this;
   } else {
-    foreach ( Content *c, d_ptr->contents ) {
+    foreach ( Content *c, d_ptr->contents() ) {
       if ( ( ret = c->textContent() ) != 0 ) {
         break;
       }
@@ -447,10 +475,10 @@ Content *Content::textContent()
 Content::List Content::attachments( bool incAlternatives )
 {
   List attachments;
-  if ( d_ptr->contents.isEmpty() ) {
+  if ( d_ptr->contents().isEmpty() ) {
     attachments.append( this );
   } else {
-    foreach ( Content *c, d_ptr->contents ) {
+    foreach ( Content *c, d_ptr->contents() ) {
       if ( !incAlternatives &&
            c->contentType()->category() == Headers::CCalternativePart ) {
         continue;
@@ -471,15 +499,18 @@ Content::List Content::attachments( bool incAlternatives )
 
 Content::List Content::contents() const
 {
-  return d_ptr->contents;
+  return d_ptr->contents();
 }
 
 void Content::addContent( Content *c, bool prepend )
 {
   Q_D( Content );
 
+  // This method makes no sense for encapsulated messages
+  Q_ASSERT( !bodyIsMessage() );
+
   // If this message is single-part; make it multipart first.
-  if( d->contents.isEmpty() && !contentType()->isMultipart() ) {
+  if( d->multipartContents.isEmpty() && !contentType()->isMultipart() ) {
     // The current body will be our first sub-Content.
     Content *main = new Content( this );
 
@@ -506,7 +537,7 @@ void Content::addContent( Content *c, bool prepend )
     d->body.clear();
 
     // Add the subcontent.
-    d->contents.append( main );
+    d->multipartContents.append( main );
 
     // Convert this content to "multipart/mixed".
     Headers::ContentType *ct = contentType();
@@ -518,9 +549,9 @@ void Content::addContent( Content *c, bool prepend )
 
   // Add the new content.
   if( prepend ) {
-    d->contents.prepend( c );
+    d->multipartContents.prepend( c );
   } else {
-    d->contents.append( c );
+    d->multipartContents.append( c );
   }
 
   if( c->parent() != this ) {
@@ -532,9 +563,15 @@ void Content::addContent( Content *c, bool prepend )
 void Content::removeContent( Content *c, bool del )
 {
   Q_D( Content );
-  Q_ASSERT( d->contents.contains( c ) );
+  if ( d->multipartContents.isEmpty() || !d->multipartContents.contains( c ) ) {
+    return;
+  }
 
-  d->contents.removeAll( c );
+  // This method makes no sense for encapsulated messages.
+  // Should be covered by the above assert already, though.
+  Q_ASSERT( !bodyIsMessage() );
+
+  d->multipartContents.removeAll( c );
   if ( del ) {
     delete c;
   } else {
@@ -542,8 +579,8 @@ void Content::removeContent( Content *c, bool del )
   }
 
   // If only one content is left, turn this content into a single-part.
-  if( d->contents.count() == 1 ) {
-    Content *main = d->contents.first();
+  if( d->multipartContents.count() == 1 ) {
+    Content *main = d->multipartContents.first();
 
     // Move all headers from the old subcontent to ourselves.
     // NOTE: This also sets the new Content-Type.
@@ -557,12 +594,16 @@ void Content::removeContent( Content *c, bool del )
 
     // Delete the old subcontent.
     delete main;
-    d->contents.clear();
+    d->multipartContents.clear();
   }
 }
 
 void Content::changeEncoding( Headers::contentEncoding e )
 {
+  // This method makes no sense for encapsulated messages, they are always 7bit
+  // encoded.
+  Q_ASSERT( !bodyIsMessage() );
+
   Headers::ContentTransferEncoding *enc = contentTransferEncoding();
   if( enc->encoding() == e ) {
     // Nothing to do.
@@ -602,17 +643,22 @@ void Content::toStream( QTextStream &ts, bool scrambleFromLines )
 
 Headers::Generic *Content::getNextHeader( QByteArray &head )
 {
-  return nextHeader( head );
+  return d_ptr->nextHeader( head );
 }
 
 Headers::Generic *Content::nextHeader( QByteArray &head )
 {
-  Headers::Base *header = HeaderParsing::extractFirstHeader( head );
+  return d_ptr->nextHeader( head );
+}
+
+Headers::Generic *ContentPrivate::nextHeader( QByteArray &_head )
+{
+  Headers::Base *header = HeaderParsing::extractFirstHeader( _head );
   if ( !header ) {
     return 0;
   }
   // Convert it from the real class to Generic.
-  Headers::Generic *ret = new Headers::Generic( header->type(), this );
+  Headers::Generic *ret = new Headers::Generic( header->type(), q_ptr );
   ret->from7BitString( header->as7BitString() );
   return ret;
 }
@@ -692,7 +738,8 @@ int Content::size()
   int ret = d_ptr->body.length();
 
   if ( contentTransferEncoding()->encoding() == Headers::CEbase64 ) {
-    return ret * 3 / 4; //base64 => 6 bit per byte
+    KMime::Codec *codec = KMime::Codec::codecForName( "base64" );
+    return codec->maxEncodedSizeFor(ret);
   }
 
   // Not handling quoted-printable here since that requires actually
@@ -707,10 +754,13 @@ int Content::storageSize() const
   const Q_D(Content);
   int s = d->head.size();
 
-  if ( d->contents.isEmpty() ) {
+  if ( d->contents().isEmpty() ) {
     s += d->body.size();
   } else {
-    foreach ( Content *c, d->contents ) {
+
+    // FIXME: This should take into account the boundary headers that are added in
+    //        encodedContent!
+    foreach ( Content *c, d->contents() ) {
       s += c->storageSize();
     }
   }
@@ -727,7 +777,7 @@ int Content::lineCount() const
   }
   ret += d->body.count( '\n' );
 
-  foreach ( Content *c, d->contents ) {
+  foreach ( Content *c, d->contents() ) {
     ret += c->lineCount();
   }
 
@@ -789,7 +839,7 @@ void Content::setDefaultCharset( const QByteArray &cs )
 {
   d_ptr->defaultCS = KMime::cachedCharset( cs );
 
-  foreach ( Content *c, d_ptr->contents ) {
+  foreach ( Content *c, d_ptr->contents() ) {
     c->setDefaultCharset( cs );
   }
 
@@ -807,7 +857,7 @@ void Content::setForceDefaultCharset( bool b )
 {
   d_ptr->forceDefaultCS = b;
 
-  foreach ( Content *c, d_ptr->contents ) {
+  foreach ( Content *c, d_ptr->contents() ) {
     c->setForceDefaultCharset( b );
   }
 
@@ -823,8 +873,8 @@ Content * KMime::Content::content( const ContentIndex &index ) const
   }
   ContentIndex idx = index;
   unsigned int i = idx.pop() - 1; // one-based -> zero-based index
-  if ( i < (unsigned int)d_ptr->contents.size() ) {
-    return d_ptr->contents[i]->content( idx );
+  if ( i < (unsigned int)d_ptr->contents().size() ) {
+    return d_ptr->contents()[i]->content( idx );
   } else {
     return 0;
   }
@@ -832,15 +882,15 @@ Content * KMime::Content::content( const ContentIndex &index ) const
 
 ContentIndex KMime::Content::indexForContent( Content * content ) const
 {
-  int i = d_ptr->contents.indexOf( content );
+  int i = d_ptr->contents().indexOf( content );
   if ( i >= 0 ) {
     ContentIndex ci;
     ci.push( i + 1 ); // zero-based -> one-based index
     return ci;
   }
   // not found, we need to search recursively
-  for ( int i = 0; i < d_ptr->contents.size(); ++i ) {
-    ContentIndex ci = d_ptr->contents[i]->indexForContent( content );
+  for ( int i = 0; i < d_ptr->contents().size(); ++i ) {
+    ContentIndex ci = d_ptr->contents()[i]->indexForContent( content );
     if ( ci.isValid() ) {
       // found it
       ci.push( i + 1 ); // zero-based -> one-based index
@@ -852,29 +902,33 @@ ContentIndex KMime::Content::indexForContent( Content * content ) const
 
 bool Content::isTopLevel() const
 {
-  return false;
+  return d_ptr->parent == 0;
 }
 
-void Content::setParent( Content* parent )
+void Content::setParent( Content *parent )
 {
-  //make sure the Content is only in the contents list of one parent object
+  // Make sure the Content is only in the contents list of one parent object
   Content *oldParent = d_ptr->parent;
-  if ( oldParent && oldParent->contents().contains( this ) ) {
-    oldParent->removeContent( this );
+  if ( oldParent ) {
+    if ( !oldParent->contents().isEmpty() && oldParent->contents().contains( this ) ) {
+      oldParent->removeContent( this );
+    }
   }
 
   d_ptr->parent = parent;
-  if ( parent && !parent->contents().contains( this ) ) {
-    parent->addContent( this );
+  if ( parent ) {
+    if ( !parent->contents().isEmpty() && !parent->contents().contains( this ) ) {
+      parent->addContent( this );
+    }
   }
 }
 
-Content* Content::parent() const
+Content *Content::parent() const
 {
   return d_ptr->parent;
 }
 
-Content* Content::topLevel() const
+Content *Content::topLevel() const
 {
   Content *top = const_cast<Content*>(this);
   Content *c = parent();
@@ -896,6 +950,23 @@ ContentIndex Content::index() const
   return indexForContent( const_cast<Content*>(this)  );
 }
 
+Message::Ptr Content::bodyAsMessage() const
+{
+  if ( bodyIsMessage() && d_ptr->bodyAsMessage ) {
+    return d_ptr->bodyAsMessage;
+  } else {
+    return Message::Ptr();
+  }
+}
+
+bool Content::bodyIsMessage() const
+{
+  // Use const_case here to work around API issue that neither header() nor hasHeader() are
+  // const, even though they should be
+  return const_cast<Content*>( this )->header<Headers::ContentType>( false ) &&
+         const_cast<Content*>( this )->header<Headers::ContentType>( true )
+                 ->mimeType().toLower() == "message/rfc822";
+}
 
 // @cond PRIVATE
 #define kmime_mk_header_accessor( type, method ) \
@@ -914,6 +985,19 @@ kmime_mk_header_accessor( ContentID, contentID )
 // @endcond
 
 
+void ContentPrivate::clearBodyMessage()
+{
+  bodyAsMessage.reset();
+}
+
+Content::List ContentPrivate::contents() const
+{
+  Q_ASSERT( multipartContents.isEmpty() || !bodyAsMessage );
+  if ( bodyAsMessage )
+    return Content::List() << bodyAsMessage.get();
+  else
+    return multipartContents;
+}
 
 bool ContentPrivate::parseUuencoded()
 {
@@ -941,27 +1025,27 @@ bool ContentPrivate::parseUuencoded()
     q->contentTransferEncoding()->clear(); // 7Bit, decoded.
 
     // Add the plain text part first.
-    Q_ASSERT( contents.count() == 0 );
+    Q_ASSERT( multipartContents.count() == 0 );
     {
       Content *c = new Content( q );
       c->contentType()->setMimeType( "text/plain" );
       c->contentTransferEncoding()->setEncoding( Headers::CE7Bit );
       c->setBody( uup.textPart() );
-      contents.append( c );
+      multipartContents.append( c );
     }
 
     // Now add each of the binary parts as sub-Contents.
     for( int i = 0; i < uup.binaryParts().count(); ++i ) {
       Content *c = new Content( q );
       c->contentType()->setMimeType( uup.mimeTypes().at( i ) );
-      c->contentType()->setName( uup.filenames().at( i ), QByteArray( /*charset*/ ) );
+      c->contentType()->setName( QLatin1String( uup.filenames().at( i ) ), QByteArray( /*charset*/ ) );
       c->contentTransferEncoding()->setEncoding( Headers::CEuuenc );
       c->contentTransferEncoding()->setDecoded( false );
       c->contentDisposition()->setDisposition( Headers::CDattachment );
-      c->contentDisposition()->setFilename( uup.filenames().at( i ) );
+      c->contentDisposition()->setFilename( QLatin1String( uup.filenames().at( i ) ) );
       c->setBody( uup.binaryParts().at( i ) );
       c->changeEncoding( Headers::CEbase64 ); // Convert to base64.
-      contents.append( c );
+      multipartContents.append( c );
     }
   }
 
@@ -995,26 +1079,26 @@ bool ContentPrivate::parseYenc()
     q->contentTransferEncoding()->clear(); // 7Bit, decoded.
 
     // Add the plain text part first.
-    Q_ASSERT( contents.count() == 0 );
+    Q_ASSERT( multipartContents.count() == 0 );
     {
       Content *c = new Content( q );
       c->contentType()->setMimeType( "text/plain" );
       c->contentTransferEncoding()->setEncoding( Headers::CE7Bit );
       c->setBody( yenc.textPart() );
-      contents.append( c );
+      multipartContents.append( c );
     }
 
     // Now add each of the binary parts as sub-Contents.
     for ( int i=0; i<yenc.binaryParts().count(); i++ ) {
       Content *c = new Content( q );
       c->contentType()->setMimeType( yenc.mimeTypes().at( i ) );
-      c->contentType()->setName( yenc.filenames().at( i ), QByteArray( /*charset*/ ) );
+      c->contentType()->setName( QLatin1String( yenc.filenames().at( i ) ), QByteArray( /*charset*/ ) );
       c->contentTransferEncoding()->setEncoding( Headers::CEbinary );
       c->contentDisposition()->setDisposition( Headers::CDattachment );
-      c->contentDisposition()->setFilename( yenc.filenames().at( i ) );
+      c->contentDisposition()->setFilename( QLatin1String( yenc.filenames().at( i ) ) );
       c->setBody( yenc.binaryParts().at( i ) ); // Yenc bodies are binary.
       c->changeEncoding( Headers::CEbase64 ); // Convert to base64.
-      contents.append( c );
+      multipartContents.append( c );
     }
   }
 
@@ -1033,7 +1117,10 @@ bool ContentPrivate::parseMultipart()
   if( !mpp.parse() ) {
     return false; // Parsing failed.
   }
-   
+
+  preamble = mpp.preamble();
+  epilogue = mpp.epilouge();
+
   // Determine the category of the subparts (used in attachments()).
   Headers::contentCategory cat;
   if( ct->isSubtype( "alternative" ) ) {
@@ -1043,7 +1130,7 @@ bool ContentPrivate::parseMultipart()
   }
 
   // Create a sub-Content for every part.
-  Q_ASSERT( contents.isEmpty() );
+  Q_ASSERT( multipartContents.isEmpty() );
   body.clear();
   QList<QByteArray> parts = mpp.parts();
   foreach( const QByteArray &part, mpp.parts() ) {
@@ -1052,7 +1139,7 @@ bool ContentPrivate::parseMultipart()
     c->setFrozen( frozen );
     c->parse();
     c->contentType()->setCategory( cat );
-    contents.append( c );
+    multipartContents.append( c );
   }
 
   return true; // Parsing successful.
