@@ -84,6 +84,11 @@ AddTransportDialog::AddTransportDialog( QWidget *parent )
     setButtons( Ok|Cancel );
     enableButtonOk( false );
     setButtonText( Ok, i18nc( "create and configure a mail transport", "Create and Configure" ) );
+
+#ifdef KDEPIM_MOBILE_UI
+    d->ui.descLabel->hide();
+    d->ui.setDefault->hide();
+#endif
   }
 
   // Populate type list.
@@ -102,8 +107,8 @@ AddTransportDialog::AddTransportDialog( QWidget *parent )
       this, SLOT(typeListClicked()) );
   connect( d->ui.typeListView, SIGNAL(itemSelectionChanged()),
       this, SLOT(typeListClicked()) );
-  connect( d->ui.typeListView, SIGNAL(doubleClicked(const QModelIndex &) ),
-           this, SLOT(accept() ) );
+  connect( d->ui.typeListView, SIGNAL(doubleClicked(QModelIndex)),
+           this, SLOT(accept()) );
 }
 
 AddTransportDialog::~AddTransportDialog()
@@ -136,9 +141,11 @@ void AddTransportDialog::accept()
   if( TransportManager::self()->configureTransport( transport, this ) ) {
     // The user clicked OK and the transport settings were saved.
     TransportManager::self()->addTransport( transport );
+#ifndef KDEPIM_MOBILE_UI    
     if( d->ui.setDefault->isChecked() ) {
       TransportManager::self()->setDefaultTransport( transport->id() );
     }
+#endif    
     KDialog::accept();
   }
 }
