@@ -18,6 +18,7 @@
 */
 
 #include "transport.h"
+#include "transport_p.h"
 #include "legacydecrypt.h"
 #include "mailtransport_defs.h"
 #include "transportmanager.h"
@@ -38,22 +39,6 @@
 using namespace MailTransport;
 using namespace KWallet;
 
-/**
- * Private class that helps to provide binary compatibility between releases.
- * @internal
- */
-class TransportPrivate
-{
-  public:
-    TransportType transportType;
-    QString password;
-    bool passwordLoaded;
-    bool passwordDirty;
-    bool storePasswordInFile;
-    bool needsWalletMigration;
-    QString oldName;
-};
-
 Transport::Transport( const QString &cfgGroup ) :
     TransportBase( cfgGroup ), d( new TransportPrivate )
 {
@@ -62,6 +47,7 @@ Transport::Transport( const QString &cfgGroup ) :
   d->passwordDirty = false;
   d->storePasswordInFile = false;
   d->needsWalletMigration = false;
+  d->passwordNeedsUpdateFromWallet = false;
   readConfig();
 }
 
@@ -201,7 +187,25 @@ void Transport::usrReadConfig()
   }
 
   // we have everything we need
-  if ( !storePassword() || d->passwordLoaded ) {
+  if ( !storePassword() ) {
+    return;
+  }
+
+  if ( d->passwordLoaded ) {
+    if ( d->passwordNeedsUpdateFromWallet ) {
+      d->passwordNeedsUpdateFromWallet = false;
+      // read password if wallet is open, defer otherwise
+      if ( Wallet::isOpen( Wallet::NetworkWallet() ) ) {
+        // Don't read the password right away because this can lead
+        // to reentrancy problems in KDBusServiceStarter when an application
+        // run in Kontact creates the transports (due to a QEventLoop in the
+        // synchronous KWallet openWallet call).
+        QTimer::singleShot( 0, this, SLOT(readPassword()) );
+      } else {
+        d->passwordLoaded = false;
+      }
+    }
+
     return;
   }
 
@@ -291,12 +295,17 @@ void Transport::readPassword()
     kDebug() << "migrating password from kmail wallet";
     KWallet::Wallet *wallet = TransportManager::self()->wallet();
     if ( wallet ) {
+      QString pwd;
       wallet->setFolder( KMAIL_WALLET_FOLDER );
-      wallet->readPassword( QString::fromLatin1( "transport-%1" ).arg( id() ), d->password );
+      if ( wallet->readPassword( QString::fromLatin1( "transport-%1" ).arg( id() ), pwd ) == 0 ) {
+        setPassword( pwd );
+        writeConfig();
+      } else {
+        d->password.clear();
+        d->passwordLoaded = false;
+      }
       wallet->removeEntry( QString::fromLatin1( "transport-%1" ).arg( id() ) );
       wallet->setFolder( WALLET_FOLDER );
-      d->passwordDirty = true;
-      writeConfig();
     }
     return;
   }
@@ -304,7 +313,13 @@ void Transport::readPassword()
   // finally try to open the wallet and read the password
   KWallet::Wallet *wallet = TransportManager::self()->wallet();
   if ( wallet ) {
-    wallet->readPassword( QString::number( id() ), d->password );
+    QString pwd;
+    if ( wallet->readPassword( QString::number( id() ), pwd ) == 0 ) {
+      setPassword( pwd );
+    } else {
+      d->password.clear();
+      d->passwordLoaded = false;
+    }
   }
 }
 
